// H-Bridge.js

(function(ext) {
    var device = null;
    var _rxBuf = [];

    // Sensor states:
    var ports = {
        Port1: 1,
        Port2: 2,
        Port3: 3,
        Port4: 4,
        Port5: 5,
        Port6: 6,
        Port7: 7,
        Port8: 8,
        Port9: 9,
        Port10: 10,
		M1:9,
		M2:10,
		
		Motor_A: 9,
		Motor_B: 10,
		A_and_B: 255
    };
    var button_keys = {
		"key1":1,
		"key2":2,
		"key3":3,
		"key4":4
	};
	var slots = {
		Slot1:1,
		Slot2:2
	};
	var switchStatus = {
		On:1,
		Off:0
	};
	var shutterStatus = {
		Press:1,
		Release:0,
		'Focus On':3,
		'Focus Off':2
	};
	var axis = {
		'X-Axis':1,
		'Y-Axis':2,
		'Z-Axis':3
	}
    var inputs = {
        slider: 0,
        light: 0,
        sound: 0,
        button: 0,
        'resistance-A': 0,
        'resistance-B': 0,
        'resistance-C': 0,
        'resistance-D': 0
    };
    function checkPortAndSlot(port, slot, sensor){
    	if((port == 4 || port == 6) && slot == 1){
			interruptThread(sensor + " not support Slot1 on Port" + port);
			return true;
		}
		return false;
    }
	var values = {};
	var indexs = [];
	var versionIndex = 0xFA;

	ext.bothMotorsConfigureAs = function(typeOfDriver) {
		var byte_1 = 0; // which motor
		var byte_2 = 0; // type of motor driver
		var byte_3 = 0; // pin1
		var byte_4 = 0; // pin2
		var byte_5 = 0; // option
		if (typeOfDriver=="MCORE") {
			byte_1 = 9;
			byte_2 = 1;
			byte_3 = 7;
			byte_4 = 6;
			byte_5 = 1;
			runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
			byte_1 = 10;
			byte_2 = 1;
			byte_3 = 4;
			byte_4 = 5;
			byte_5 = 0;
			runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
		} else if (typeOfDriver=="AURIGA") {
			byte_1 = 9;
			byte_2 = 2;
			byte_3 = 47;
			byte_4 = 46;
			byte_5 = 10;
			runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
			byte_1 = 10;
			byte_2 = 2;
			byte_3 = 48;
			byte_4 = 49;
			byte_5 = 11;
			runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);			
		} else if (typeOfDriver=="HG7881") {
			byte_1 = 9;
			byte_2 = 3;
			byte_3 = 3;
			byte_4 = 9;
			byte_5 = 0;
			runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
			byte_1 = 10;
			byte_2 = 3;
			byte_3 = 10;
			byte_4 = 11;
			byte_5 = 0;
			runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);			
		}
    };

	ext.configureORION = function(motorIndex,pin1,pin2,turnOpt) {
		var byte_1 = motorIndex; // which motor
		var byte_2 = 1; // MCORE-like type of motor driver
		var byte_3 = pin1; // pin1
		var byte_4 = pin2; // pin2
		var byte_5 = turnOpt; // direct/opposite
		if(typeof byte_1=="string"){
			byte_1 = ports[byte_1];
		}
		if (byte_5=="direct") {
			byte_5 = 0;
		} else if (byte_5=="opposite") {
			byte_5 = 1;
		}
		runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
    };

	ext.configureAURIGA = function(motorIndex,pin1,pin2,pinPWM) {
		var byte_1 = motorIndex; // which motor
		var byte_2 = 2; // AURIGA-like type of motor driver
		var byte_3 = pin1; // pin1
		var byte_4 = pin2; // pin2
		var byte_5 = pinPWM; // analog write pin
		if(typeof byte_1=="string"){
			byte_1 = ports[byte_1];
		}
		runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
    };

	ext.configureHG7881 = function(motorIndex,pin1,pin2,driverLogic) {
		var byte_1 = motorIndex; // which motor
		var byte_2 = 3; // HG7881-like type of motor driver
		var byte_3 = pin1; // pin1
		var byte_4 = pin2; // pin2
		var byte_5 = 0; // type of logic, harmless/harmless
		if(typeof byte_1=="string"){
			byte_1 = ports[byte_1];
		}
		if (driverLogic=="HD") {
			byte_5 = 1; // harmless/dangerous
		} else if (driverLogic=="DH") {
			byte_5 = 2; // dangerous/harmless
		} else if (driverLogic=="DD") {
			byte_5 = 3; // dangerous/dangerous
		}
		runPackage(29,byte_1,byte_2,byte_3,byte_4,byte_5);
    };

	ext.moveHBridgeRobot = function(movDirection,movSpeed) {
		var s1 = 0;
		var s2 = 0;
		if(movDirection=="Forward"){
			s1 = -movSpeed;
			s2 = movSpeed;
		}else if(movDirection=="Backward"){
			s1 = movSpeed;
			s2 = -movSpeed;
		}else if(movDirection=="Left"){
			s1 = movSpeed;
			s2 = movSpeed;
		}else if(movDirection=="Right"){
			s1 = -movSpeed;
			s2 = -movSpeed;
		}
        if (s1>255) {s1=255} else if (s1<-255) {s1=-255}
        if (s2>255) {s2=255} else if (s2<-255) {s2=-255}
		runPackage(5,short2array(s1),short2array(s2));
    };
    
	ext.runHBridgeMotors = function(leftMotorSpeed,rightMotorSpeed) {
		var s1 = -leftMotorSpeed;
		var s2 = rightMotorSpeed;
        if (s1>255) {s1=255} else if (s1<-255) {s1=-255}
        if (s2>255) {s2=255} else if (s2<-255) {s2=-255}
        runPackage(5,short2array(s1),short2array(s2));
    };
    
	ext.singleHBridgeMotor = function(motorIndex,motorSpeed) {
		var wM = motorIndex;
		var spd = motorSpeed;
		if(typeof wM=="string"){
			wM = ports[wM];
		}
        if (wM == 9) {spd = -spd;}
		if (spd>255) {spd=255;} else if (spd<-255) {spd=-255;}
		runPackage(10,wM,short2array(spd));
    };

	ext.stopHBridgeMotorOrBoth = function(brakeStyle,whichMotors) {
		var wM = whichMotors;
		var s = 0;
		if (wM=="A_and_B") {
			wM = 255;
		} else if (typeof wM=="string") {
			wM = ports[wM];
		}
		if (brakeStyle=="free") {
			s=32763;
		} else if (brakeStyle=="BRAKE") {
			s=32764;
		} else if (brakeStyle=="release") {
			s=32766;
		} else if (brakeStyle=="default") {
			s=0;
		}
		if (wM==255) {
			runPackage(5,short2array(s),short2array(s));
		} else {
			runPackage(10,wM,short2array(s));
		}
	};
    
	ext.unattachHBridgeMotorOrBoth = function(whichMotors) {
		var wM = whichMotors;
		var s = 32767;
		if (wM=="A_and_B") {
			wM = 255;
		} else if (typeof wM=="string") {
			wM = ports[wM];
		}
		if (wM==255) {
//			runPackage(5,short2array(s),short2array(s));
			wM = 9;
			runPackage(10,wM,short2array(s));
			wM = 10;
			runPackage(10,wM,short2array(s));
		} else {
			runPackage(10,wM,short2array(s));
		}
	};
    
	
	function sendPackage(argList, type){
		var bytes = [0xff, 0x55, 0, 0, type];
		for(var i=0;i<argList.length;++i){
			var val = argList[i];
			if(val.constructor == "[class Array]"){
				bytes = bytes.concat(val);
			}else{
				bytes.push(val);
			}
		}
		bytes[2] = bytes.length - 3;
		device.send(bytes);
	}
	
	function runPackage(){
		sendPackage(arguments, 2);
	}
	function getPackage(){
		var nextID = arguments[0];
		Array.prototype.shift.call(arguments);
		sendPackage(arguments, 1);
	}
    
    var inputArray = [];
	var _isParseStart = false;
	var _isParseStartIndex = 0;
    function processData(bytes) {
		var len = bytes.length;
		if(_rxBuf.length>30){
			_rxBuf = [];
		}
		for(var index=0;index<bytes.length;index++){
			var c = bytes[index];
			_rxBuf.push(c);
			if(_rxBuf.length>=2){
				if(_rxBuf[_rxBuf.length-1]==0x55 && _rxBuf[_rxBuf.length-2]==0xff){
					_isParseStart = true;
					_isParseStartIndex = _rxBuf.length-2;
				}
				if(_rxBuf[_rxBuf.length-1]==0xa && _rxBuf[_rxBuf.length-2]==0xd&&_isParseStart){
					_isParseStart = false;
					
					var position = _isParseStartIndex+2;
					var extId = _rxBuf[position];
					position++;
					var type = _rxBuf[position];
					position++;
					//1 byte 2 float 3 short 4 len+string 5 double
					var value;
					switch(type){
						case 1:{
							value = _rxBuf[position];
							position++;
						}
							break;
						case 2:{
							value = readFloat(_rxBuf,position);
							position+=4;
						}
							break;
						case 3:{
							value = readInt(_rxBuf,position,2);
							position+=2;
						}
							break;
						case 4:{
							var l = _rxBuf[position];
							position++;
							value = readString(_rxBuf,position,l);
						}
							break;
						case 5:{
							value = readDouble(_rxBuf,position);
							position+=4;
						}
							break;
						case 6:
							value = readInt(_rxBuf,position,4);
							position+=4;
							break;
					}
					if(type<=6){
						responseValue(extId,value);
					}else{
						responseValue();
					}
					_rxBuf = [];
				}
			} 
		}
    }
	function readFloat(arr,position){
		var f= [arr[position],arr[position+1],arr[position+2],arr[position+3]];
		return parseFloat(f);
	}
	function readInt(arr,position,count){
		var result = 0;
		for(var i=0; i<count; ++i){
			result |= arr[position+i] << (i << 3);
		}
		return result;
	}
	function readDouble(arr,position){
		return readFloat(arr,position);
	}
	function readString(arr,position,len){
		var value = "";
		for(var ii=0;ii<len;ii++){
			value += String.fromCharCode(_rxBuf[ii+position]);
		}
		return value;
	}
    function appendBuffer( buffer1, buffer2 ) {
        return buffer1.concat( buffer2 );
    }

    // Extension API interactions
    var potentialDevices = [];
    ext._deviceConnected = function(dev) {
        potentialDevices.push(dev);

        if (!device) {
            tryNextDevice();
        }
    }

    function tryNextDevice() {
        // If potentialDevices is empty, device will be undefined.
        // That will get us back here next time a device is connected.
        device = potentialDevices.shift();
        if (device) {
            device.open({ stopBits: 0, bitRate: 115200, ctsFlowControl: 0 }, deviceOpened);
        }
    }

    var watchdog = null;
    function deviceOpened(dev) {
        if (!dev) {
            // Opening the port failed.
            tryNextDevice();
            return;
        }
        device.set_receive_handler('H-Bridge',processData);
    };

    ext._deviceRemoved = function(dev) {
        if(device != dev) return;
        device = null;
    };

    ext._shutdown = function() {
        if(device) device.close();
        device = null;
    };

    ext._getStatus = function() {
        if(!device) return {status: 1, msg: 'H-Bridge disconnected'};
        if(watchdog) return {status: 1, msg: 'Probing for H-Bridge'};
        return {status: 2, msg: 'H-Bridge connected'};
    }

    var descriptor = {};
	ScratchExtensions.register('H-Bridge', descriptor, ext, {type: 'serial'});
})({});
